==> falak-0.1.31.tgz: package/framework/server/server.js <==
const express = require('express');
const app = express();
const { fs } = require('./../bootstrap/modules');
const { sassCompiler } = require('./../bootstrap');

let originalHtml = fs.get(FRAMEWORK_ROOT_PATH + '/outputs/html/index.dev.html');

function setApp(request) {
    for (let appName in config.apps) {
        let appConfig = config.apps[appName];
        if (request.path.startsWith(appConfig.path)) {
            global.appConfig = appConfig;
            appConfig.name = appName;
            setCurrentLocale(appConfig, request);
            break;
        }
    }
}

function getDirectionOf(localeCode) {
    return config.locales[localeCode].direction;
}

function setCurrentLocale(appConfig, request) {
    let oldLocale = CURRENT_LOCALE; 
    if (request.path == appConfig.path) {
        global.CURRENT_DIRECTION = getDirectionOf(appConfig.locale);
        global.CURRENT_LOCALE = appConfig.locale;
    }

    for (let locale of appConfig.locales) {
        if (request.path.startsWith(appConfig.path.ltrim('/') + '/' + locale)) {
            global.CURRENT_LOCALE = locale;
            global.CURRENT_DIRECTION = getDirectionOf(locale);
            break;
        }
    }    

    // clear compiled sass code if the locale has changed
    if (oldLocale != CURRENT_LOCALE) {
        sassCompiler.clear(CURRENT_DIRECTION);
    }
}

app.get('*', async (request, response) => {
    if ((request.path.startsWith('/public') && ! request.path.startsWith(`/public/${APP_NAME}/css`))
        || request.path.startsWith('/src') || request.path.startsWith('/node_modules')) {
        return response.sendFile(ROOT + request.path);
    }

    setApp(request);

    if (request.path.startsWith(`/public/${APP_NAME}/css`)) {
        try {
            let compiledStyle = await sassCompiler.compile(CURRENT_DIRECTION);
            response.setHeader("Content-Type", "text/css");
            return response.send(compiledStyle);
        } catch (e) {
            sassCompiler.error(e);
        }
    }

    response.send(sendResponse());
});

function sendResponse() {
    let localeCode = CURRENT_LOCALE;

    let title = appConfig.title[localeCode];

    let response = originalHtml.replace('APP_DIR', CURRENT_DIRECTION)
        .replace('APP_TITLE', title)
        .replace('APP_NAME', appConfig.name)
        .replace('APP_LOCALE', CURRENT_LOCALE)
        .replace('APP_BASE_URL', BASE_URL)
        .replace('STYLESHEETS', getStylesheets(CURRENT_DIRECTION, appConfig.name))
        .replace('SCRIPTS_LIST', getScriptTags());

    return response;
}

function getScriptTags() {
    // return `<script src="public/${APP_NAME}/js/${JS_BUNDLE_FILE_NAME}"></script>`
    return resources.jsVendor.concat(resources.jsFiles).map(filePath => {
        return `<script src="${filePath}"></script>`
    }).join('');
}

function getStylesheets(direction) {
    let stylesheets = `
        <link rel="stylesheet" dir="${direction}" id="app-style" href="/public/${APP_NAME}/css/app-${direction}.css?v=" />
    `;

    if (! Is.empty(Object.get(resources.externals, `css.${CURRENT_DIRECTION}`)) || ! Is.empty(Object.get(resources.externals, `css.common`))) {
    // if (! Is.empty(resources.externals.css[CURRENT_DIRECTION]) || ! Is.empty(resources.externals.css.common)) {
        let commonCdns = resources.externals.css.common || [],
            directionCdns = resources.externals.css[CURRENT_DIRECTION] || [],
            cdns = commonCdns.concat(directionCdns);
        stylesheets = cdns.map(stylesheet => `<link rel="stylesheet" href="${stylesheet}" />`).join("") + stylesheets;
    }

    if (resources.favicon) {
        stylesheets =  `
            <link rel="icon" type="image/x-icon" href="public/${APP_NAME}/${resources.favicon}" />
        ` + stylesheets;   
    }

    return stylesheets;
}

function sassError(error) {
    sassCompiler.error(error);
    // die();
}

async function startServer(command) {
    echo.sameLine(cli.cyan('Compiling sass files...'));
    await sassCompiler.compile('ltr').then().catch(sassError);
    await sassCompiler.compile('rtl').then().catch(sassError);
    echo.sameLine(cli.green('Sass files have been compiled successfully.'));

    let port = command.options.port,
        scriptUrl = SCRIPT_URL;

    if (port && port != process.env.PORT) {
        scriptUrl = scriptUrl.replace(process.env.PORT, port);        
    } else {
        port = process.env.PORT;
    }

    app.listen(port, (err) => {
        echo.sameLine(cli.green(`Start browsing the application from ${cli.cyan(scriptUrl)}`));
        echo();
        if (command.open) {
            let openOptions = {};

            if (command.browser) {
                openOptions.app = command.browser;
            } else if (config.browser != 'default') {
                openOptions.app = config.browser;
            }

            const open = require('opn');
            open(scriptUrl, openOptions);
        }
    });
}

module.exports = {
    start: startServer,
};

==> falak-0.1.33.tgz: package/framework/server/server.js <==
const express = require('express');
const app = express();
const { fs } = require('./../bootstrap/modules');
const { sassCompiler } = require('./../bootstrap');

let originalHtml = fs.get(FRAMEWORK_ROOT_PATH + '/outputs/html/index.dev.html');

function setApp(request) {
    for (let appName in config.apps) {
        let appConfig = config.apps[appName];
        if (request.path.startsWith(appConfig.path)) {
            global.appConfig = appConfig;
            appConfig.name = appName;
            setCurrentLocale(appConfig, request);
            break;
        }
    }
}

function getDirectionOf(localeCode) {
    return config.locales[localeCode].direction;
}

function setCurrentLocale(appConfig, request) {
    let oldLocale = CURRENT_LOCALE; 
    if (request.path == appConfig.path) {
        global.CURRENT_DIRECTION = getDirectionOf(appConfig.locale);
        global.CURRENT_LOCALE = appConfig.locale;
    }

    for (let locale of appConfig.locales) {
        if (request.path.startsWith(appConfig.path.ltrim('/') + '/' + locale)) {
            global.CURRENT_LOCALE = locale;
            global.CURRENT_DIRECTION = getDirectionOf(locale);
            break;
        }
    }    

    // clear compiled sass code if the locale has changed
    if (oldLocale != CURRENT_LOCALE) {
        sassCompiler.clear(CURRENT_DIRECTION);
    }
}

app.get('*', async (request, response) => {
    if ((request.path.startsWith('/public') && ! request.path.startsWith(`/public/${APP_NAME}/css`))
        || request.path.startsWith('/src') || request.path.startsWith('/node_modules')) {
        return response.sendFile(ROOT + request.path);
    }

    setApp(request);

    if (request.path.startsWith(`/public/${APP_NAME}/css`)) {
        try {
            let compiledStyle = await sassCompiler.compile(CURRENT_DIRECTION);
            response.setHeader("Content-Type", "text/css");
            return response.send(compiledStyle);
        } catch (e) {
            sassCompiler.error(e);
        }
    }

    response.send(sendResponse());
});

function sendResponse() {
    let localeCode = CURRENT_LOCALE;

    let title = appConfig.title[localeCode];

    let response = originalHtml.replace('APP_DIR', CURRENT_DIRECTION)
        .replace('APP_TITLE', title)
        .replace('APP_NAME', appConfig.name)
        .replace('APP_LOCALE', CURRENT_LOCALE)
        .replace('APP_BASE_URL', BASE_URL)
        .replace('STYLESHEETS', getStylesheets(CURRENT_DIRECTION, appConfig.name))
        .replace('SCRIPTS_LIST', getScriptTags());

    return response;
}

function getScriptTags() {
    let cdnFiles = resources.externals.js || [];
    return cdnFiles.concat(resources.jsVendor).concat(resources.jsFiles).map(filePath => {
        return `<script src="${filePath}"></script>`;
    }).join('');
}

function getStylesheets(direction) {
    let stylesheets = `
        <link rel="stylesheet" dir="${direction}" id="app-style" href="/public/${APP_NAME}/css/app-${direction}.css?v=" />
    `;

    if (! Is.empty(Object.get(resources.externals, `css.${CURRENT_DIRECTION}`)) || ! Is.empty(Object.get(resources.externals, `css.common`))) {
    // if (! Is.empty(resources.externals.css[CURRENT_DIRECTION]) || ! Is.empty(resources.externals.css.common)) {
        let commonCdns = resources.externals.css.common || [],
            directionCdns = resources.externals.css[CURRENT_DIRECTION] || [],
            cdns = commonCdns.concat(directionCdns);
        stylesheets = cdns.map(stylesheet => `<link rel="stylesheet" href="${stylesheet}" />`).join("") + stylesheets;
    }

    if (resources.favicon) {
        stylesheets =  `
            <link rel="icon" type="image/x-icon" href="public/${APP_NAME}/${resources.favicon}" />
        ` + stylesheets;   
    }

    return stylesheets;
}

function sassError(error) {
    sassCompiler.error(error);
    // die();
}

async function startServer(command) {
    echo.sameLine(cli.cyan('Compiling sass files...'));
    await sassCompiler.compile('ltr').then().catch(sassError);
    await sassCompiler.compile('rtl').then().catch(sassError);
    echo.sameLine(cli.green('Sass files have been compiled successfully.'));

    let port = command.options.port,
        scriptUrl = SCRIPT_URL;

    if (port && port != process.env.PORT) {
        scriptUrl = scriptUrl.replace(process.env.PORT, port);        
    } else {
        port = process.env.PORT;
    }

    app.listen(port, (err) => {
        echo.sameLine(cli.green(`Start browsing the application from ${cli.cyan(scriptUrl)}`));
        echo();
        if (command.open) {
            let openOptions = {};

            if (command.browser) {
                openOptions.app = command.browser;
            } else if (config.browser != 'default') {
                openOptions.app = config.browser;
            }

            const open = require('opn');
            open(scriptUrl, openOptions);
        }
    });
}

module.exports = {
    start: startServer,
};
